﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Flight_Data_Recorder
{
    public partial class Main : Form
    {
        double[] CurrentData = new double[10];//this will hold the latest data retrived from FSUIPC
        int counter = 0;//this will count how many records have been saved since the start
        bool IsRecording = false;//Boolean describing if the system is currently recording data
        string path = "";//stores the path of the log file
        int GraphField = 3; //stores the current feild of data being graphed, default is set to altitude

        public Main()
        {
            InitializeComponent();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            CurrentData = FSData.GetAircraftDataArray();//retrived the latest data from the FSData Class and places them in CurrentData

            UpdateLabels();//Updates the labels on the form

            //data recording code
            if (IsRecording)//if the system is currently recording
            {
                counter++;//adds one record to the counter
                CurrentData[0] = counter;//creates the ID field for the data (number of records since start)
                DataReadWrite.WriteNewLine(path, CurrentData);//Uses the DataReadWrite Class to store a line of data in the log file
                UpdateTable();//adds a new record to the table
                UpdateChart();//adds a new record to the graph
            }

        }

        private void cmdStartStop_Click(object sender, EventArgs e)
        {
            if (cmdStartStop.Text == "Open Communications")//acting as a switch for the open/close communications button
            {
                timer1.Enabled = true;//starts the timer that retrives the data (see timer1_Tick)
                cmdStartData.Enabled = true;//enables the Start Data Recording button
                cmdStartStop.Text = "Close Communications";//sets this buttons text to close communications
            }
            else
            {
                timer1.Enabled = false;//stops the timer that retrives the data
                cmdStartStop.Text = "Open Communications";//sets this label to open
            }
        }

        private void toolStripMenuItem4_Click(object sender, EventArgs e)
        {
            timer1.Interval = 1000;//sets the interval of the update timer to 1 second
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {
            timer1.Interval = 10;//sets the interval to 1/100 of a second
        }

        private void toolStripMenuItem3_Click(object sender, EventArgs e)
        {
            timer1.Interval = 100;//sets the interval to 1/10 of a second
        }

        private void toolStripMenuItem5_Click(object sender, EventArgs e)
        {
            timer1.Interval = 2000;//sets the interval to 2 seconds
        }

        private void secondsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            timer1.Interval = 10000;//sets the interval to 10 seconds
        }

        private void cmdStartData_Click(object sender, EventArgs e)
        {
            if (cmdStartData.Text == "Start Data Collection")//acting as a swtich for the "Start/Stop Data collection button"
            {
                if (path != "")//if the stored path has no value, it will prompt the user to assign one
                {
                    //This is if the code that starts the data collection process
                    IsRecording = true;//sets the system to recording
                    cmdStartData.Text = "Stop Data Collection";//sets this label to "Stop Data..."
                    cmdStartData.BackColor = Color.Red;//Sets the backcolor to red

                    //The following code is designed to give options about recording multiple segments of data in a row

                    if (optPreserveNums.Checked)
                    {
                        //Preserve nums will keep the numbering as is (E.G. if we are at ID 5 when we clicked stop, it will record the new data with ID 6)
                        UpdateChart();
                    }
                    else if (optStartNumOver.Checked)
                    {
                        //StartNumOver will start the numbering over (E.G. If we hit stop at ID 5, it will keep the existing data and on line 6 record with ID 1)
                        WipeGraphData();//Wipes the graphs display
                        counter = 0; //sets ID back to 0
                    }
                    else if (optWipeData.Checked)
                    {
                        //WipeData will wipe all the data in the file and start over
                        WipeGraphData(); //deletes the stored graph data so it will start at 0;
                        WipeAllTableRows();//deletes all data records in the table

                        DataReadWrite.CreateFileWithHeaders(path);//creates a new file (with headers) and wipes all data in the file
                        counter = 0; //sets the counter to 0 so numbering will start over
                    }
                }
                else
                {
                    MessageBox.Show("Please go to File -> Select File Location, and create a valid file path", "You did not choose a valid file path");
                    //Shows a message prompting the user to select a log file
                    IsRecording = false;//sets the system to not record (not really needed, just a precaution to avoid errors)
                    cmdStartData.Text = "Start Data Collection";//sets this label back to start
                    cmdStartData.BackColor = Color.Green;//sets the backcolor to green
                }

            }
            else//if we are already recording
            {
                IsRecording = false;//stop collecting data
                cmdStartData.Text = "Start Data Collection";//sets the label back to start
                cmdStartData.BackColor = Color.Green;//changes this buttons label back to green
            }
        }

        public void UpdateLabels()//updates labels on the form to the data in CurrentData[]
        {
            lblAlt.Text = string.Format("Altitude = {0} feet", CurrentData[3].ToString());//writes the altitude label
            lblhead.Text = string.Format("Heading = {0} degrees", CurrentData[4].ToString());//writes the heading label
            lblbank.Text = string.Format("Bank = {0} degrees", CurrentData[5].ToString());//write the bank label
            lblpitch.Text = string.Format("Pitch = {0} degrees", CurrentData[6].ToString());//writes the pitch label
            lblspeed.Text = string.Format("Indicated Airspeed = {0} knots", CurrentData[7].ToString());//writes the speed label
            lblvertspeed.Text = string.Format("Vertical Speed = {0} feet per second", CurrentData[8].ToString());//writes the vertical speed labels

            if (CurrentData[1] < 0)//if latitude is less than zero, print degrees south and make it positive
            {
                lblLat.Text = string.Format("Latitude = {0} degrees south", (CurrentData[1] * -1).ToString());
            }
            else//if it is greater, print north
            {
                lblLat.Text = string.Format("Latitude = {0} degrees north", (CurrentData[1]).ToString());
            }

            if (CurrentData[2] < 0)//if longitude is less than 0, print west and make positive
            {
                lblLong.Text = string.Format("Longitude = {0} degrees west", (CurrentData[2] * -1).ToString());
            }
            else//otherwise print east and keep the same
            {
                lblLong.Text = string.Format("Longitude = {0} degrees east", (CurrentData[2]).ToString());
            }
        }

        private void Main_Load(object sender, EventArgs e)//form startup
        {
            UpdateLabels();//print labels with 0 data
            optStartNumOver.Checked = true;//set the default recording option to Start Numbering over
            saveFileDialog1.Title = "Select/Create File Log File";//sets the title of the save prompt dialog
            saveFileDialog1.DefaultExt = ".csv";//sets the default extension as .CSV
            saveFileDialog1.AddExtension = true;//sets the dialog to automaticlly add the extension
            saveFileDialog1.ShowDialog();//shows the file dialog
        }

        private void saveDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();//shows the file dialog
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)//runs when the user hits save on the file dialog
        {
            path = Path.GetDirectoryName(saveFileDialog1.FileName);//gets the path and puts it in string path
            path += "\\" + Path.GetFileName(saveFileDialog1.FileName);//adds a slash and the file name
            lblCurrPath.Text = "Recording to: " + path;//sets the "Recording To:" label to the current path
            DataReadWrite.CreateFileWithHeaders(path);//creates a new blank file with headers based on the path
        }

        private void openFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();//shows the open file dialog
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)//when the user clicks open in the open file dialog
        {
            path = Path.GetDirectoryName(openFileDialog1.FileName);//gets the path and puts it in string path
            path += "\\" + Path.GetFileName(openFileDialog1.FileName);//adds a slash and the file name
            lblCurrPath.Text = "Opened: " + path;//sets the "Recording To:" label to the current path

            counter = 0;//resets the ID counter
            WipeGraphData();//wipes all the data in the graph
            StartGraph();//Reads the data from the file and writes them to the graph
            StartTable();//Reads the data from the file and writes them to the table
        }

        //GRAPH CODE
        public void UpdateChart()//Adds a new point to the end of the graph
        {
            if (chart1.Series[0].Points.Count > 1)//if there is more than one point on the current graph
            {
                chart1.Series[0].Points.AddXY(chart1.Series[0].Points.Count + 1, CurrentData[GraphField]);//make a new point at the end
            }
            else
            {
                chart1.Series[0].Points.AddXY(1, CurrentData[GraphField]);//If there are no points, create the first point
            }
        }
        public void StartGraph()//Reads the data from the file and places it on the graph
        {
            int NumOfLines = GetNumberOfLinesInFile() - 1;//gets the number of line in the file and subtracts one for the headers

            if (NumOfLines != 0)//if there are more than 0 lines in the file
            {
                double[] History = new double[NumOfLines];//will store the data for a single field read from the file 
                for (int i = 1; i < NumOfLines; i++)//for each number of lines retirve the data for the current field
                {
                    History[i] = DataReadWrite.GetDataLine(i, path)[GraphField];//writes data from file into history
                }

                if (NumOfLines > 2)//if there are more than two lines in the file (headers and another line)
                {
                    History[0] = History[1];//sets the 0th value same as the 1st to avoid annoying empty 0 values
                }//I know that this isn't accurate but since the system doesn't record a 0th data point, it will make the graph look neater

                for (int i = 1; i < NumOfLines; i++)//runs for every point
                {
                    chart1.Series[0].Points.AddXY(i, History[i]);//adds a point to the graph
                }

                ChangeChartLabels(GraphField);//adds labels to the graph (E.G. "Altitude and ID") 
            }
        }
        public void ChangeChartLabels(int feild)//changes the chart's axe's labels
        {
            switch (feild)
            {
                case 3://Altitude
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Altitude (feet)";
                    break;
                case 4://heading
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Heading (degrees)";
                    break;
                case 5://Bank
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Bank (degrees)";
                    break;
                case 6://Pitch
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Pitch (feet)";
                    break;
                case 7://Airspeed
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Indicated Airspeed (knots)";
                    break;
                case 8://Vertical Speed
                    chart1.ChartAreas[0].AxisX.Title = "ID/Time";
                    chart1.ChartAreas[0].AxisY.Title = "Vertical Speed (Feet per Second)";
                    break;
            }
        }
        public void WipeGraphData()//Wipes all the graphs points
        {
            int NumberOfRecords = chart1.Series[0].Points.Count - 1;//find the number of records on the graph currently (how many we need to delete)
            //subtracts one to account for starting the list at 0
            for (int i = NumberOfRecords; i > 0; i--)//for each point on the graph (runs backwards)
            {
                chart1.Series[0].Points.RemoveAt(i);//removes the point
            }
        }

        private void optAltGraph_CheckedChanged(object sender, EventArgs e)
        {
            WipeGraphData();//wipes the graph data
            GraphField = 3;//sets the current graphing field to 3 (altitude)
            StartGraph();//reads the data from the file adn graphs it
            UpdateChart();//adds a current point
        }
        private void optBankGraph_CheckedChanged(object sender, EventArgs e)
        {
            WipeGraphData();
            GraphField = 5;//sets the current graphing field to 5 (bank)
            StartGraph();
            UpdateChart();
        }
        private void optPitchGraph_CheckedChanged(object sender, EventArgs e)
        {
            WipeGraphData();
            GraphField = 6;//sets the current graphing field to 6 (pitch)
            StartGraph();
            UpdateChart();
        }
        private void optAirspdGraph_CheckedChanged(object sender, EventArgs e)
        {
            WipeGraphData();
            GraphField = 7;//sets the current graphing field to 7 (airspeed)
            StartGraph();
            UpdateChart();
        }
        private void optVertSpeedGraph_CheckedChanged(object sender, EventArgs e)
        {
            WipeGraphData();
            GraphField = 8;//sets the current graphing field to 8 (vertical speed)
            StartGraph();
            UpdateChart();
        }

        //TABLE CODE
        public void UpdateTable()//adds another row to the end of the table
        {
            int CurrRow = dataGridView1.Rows.Add();//adds a new row to the table and gets the index for that row
            for (int i = 0; i < 9; i++)//for each field (column)
            {
                dataGridView1.Rows[CurrRow].Cells[i].Value = CurrentData[i];//prints the current data of the Ith field in that cell 
            }
        }
        public void StartTable()//retrieves data from the file and prints them into the table
        {
            int NumOfLines = GetNumberOfLinesInFile() - 1;//gets the number of lines in the file (minus one for the headers)

            if (NumOfLines != 0)//Makes sure there is data in the file
            {
                double[] History = new double[9];//holds a single row of data retrived from the file
                WipeAllTableRows();//clears the table
                for (int i = 1; i < NumOfLines; i++)//for every line in the file
                {
                    History = DataReadWrite.GetDataLine(i, path);//reads the I'th row into the History array
                    int CurrRow = dataGridView1.Rows.Add(); //adds a row into the table and sets CurrRow to the index of that row
                    for (int x = 0; x < 9; x++)//runs for every column(i) in every row(x)
                    {
                        dataGridView1.Rows[CurrRow].Cells[x].Value = History[x];//prints all the data in History[] into the cells of CurrRow
                    }
                }
            }
        }
        public void WipeAllTableRows()//delets all the rows in the table
        {
            int TotalRows = dataGridView1.Rows.GetRowCount(0) - 2;//Gets the total number of rows
            for (int i = TotalRows; i >= 0; i--)//foreach row in the table (runs backwards)
            {
                dataGridView1.Rows.RemoveAt(i);//removes the Ith row
            }
            
            dataGridView1.Update();
        }

        //BROWSER/MAP CODE
        private void browserUpdate_Tick(object sender, EventArgs e)//the browser update timer
        {       
            //the map is actually a browser that points to a webpage that I built using the Google Maps API
            //Using PHP GET variables it stores the lat and long within the web address
            if (optMapURL1.Checked)//if the radio button for the first server is checked
            {
                webBrowser1.Url = new Uri(String.Format("http://www.flightmap.comoj.com/?lat={0}&long={1}&zoom={2}", CurrentData[1], CurrentData[2], sldrZoom.Value));
                //Navigates to the given address and embeds the proper variables
            }
            else//if the other radio button is checked
            {
                webBrowser1.Url = new Uri(String.Format("http://flightmap.comze.com/?lat={0}&long={1}&zoom={2}", CurrentData[1], CurrentData[2], sldrZoom.Value));
                //Navigates to the given address and embeds the proper variables
            }
        }
        private void numMapTimerUpdate_ValueChanged(object sender, EventArgs e)//map update rate numeric up/down control changed
        {
            browserUpdate.Interval = Convert.ToInt16(numMapTimerUpdate.Value * 1000);//sets the interval of the map update timer to the value of the control
        }
        private void OptMapEnable_CheckedChanged(object sender, EventArgs e)
        {
            browserUpdate_Tick(sender, e); //updates map
            browserUpdate.Enabled = true; //sets the map update timer to enabled (will regularly update)
        }
        private void optMapDisable_CheckedChanged(object sender, EventArgs e)
        {
            browserUpdate.Enabled = false;//sets the update timer to off (will not update)
        }
        private void cmdUpdateMap_Click(object sender, EventArgs e)
        {
            browserUpdate_Tick(sender, e); //updates map
        }

        //OTHER FUNCTIONS
        public int GetNumberOfLinesInFile()
        {
            var lineCount = 0;
            using (var reader = File.OpenText(path))//opens current file
            {
                while (reader.ReadLine() != null)
                {
                    lineCount++;//adds one to the line count
                }
            }
            return lineCount;
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Flight_Data_Recorder.About AboutForm = new About();//creates new instance of about form
            AboutForm.Show();//shows about form
        }

    }
}
